#CloudyDay.py

#imports
import pygame
import math
import sys
import os

from Turtle import Turtle
from Rule import RuleSystem

from Inchworm import Inchworm

from pygame.locals import *

from engine.Gameworld import Gameworld

from engine.AtlanSpriteGroup import AtlanSpriteGroup

from common.Utility import *

from Cloud import Cloud

from HeroCloud import HeroCloud

from RainDrop import RainDrop

from Rule import RuleSystem

from Turtle import Turtle

from Bug import Bug

from Vector import Vec2


import random

import math

class CloudyDay(Gameworld):
	"""
                The main game screen for the game.
                Handle the drawing, update, and input response for A Cloudy Day.
	"""

	def __init__(self):
                """
                        Do nothing, all setup is done in reset method.
                """
		pass

        def reset(self):
                """
                        Set up the game with no plants, 3 random bugs,
                        1 inchworm, no additional sounds playing,
                        a random player cloud and 5 regular clouds.
                        
                """
                #regular clouds
                self.foreClouds = []
                self.foreClouds.append(Cloud(pygame.Rect(100, 75, random.randint(50, 200), random.randint(50, 100))))
                self.foreClouds.append(Cloud(pygame.Rect(350, 0, random.randint(50, 200), random.randint(50, 200))))
                self.foreClouds.append(Cloud(pygame.Rect(500, 100, random.randint(50, 200), random.randint(50, 200))))
                self.foreClouds.append(Cloud(pygame.Rect(200, 150, random.randint(50, 200), random.randint(50, 150))))
                self.foreClouds.append(Cloud(pygame.Rect(600, 15, random.randint(50, 200), random.randint(50, 150))))


                #player character
		self.hero = HeroCloud(pygame.Rect(50, 50, 200, 50))
		#no rain
		self.drops = []
		self.raining = False
		self.rainCount = 0
		#no plants
		self.turtles = []
		#3 random bugs
		self.bugs = []
		for i in range(3):
                        self.bugs.append(Bug(pygame.Rect(random.randint(0, XRES), random.randint(0, YRES - 20), 1, 1)))
		self.windSpeed = 2.3
		self.holdLeft = False
		self.holdRight = False
		self.lightningCounter = 0
		#load sounds
		self.rainSound = pygame.mixer.Sound(load_sound("rain.ogg"))
		self.bugSound = pygame.mixer.Sound(load_sound("fly.ogg"))
		self.bugSoundCount = 100
		self.rainSoundCount = 100
		self.playingRainSound = False
		self.playingBugSound = False
		self.spacing = 10
		self.lightningCounter = 180
		#1 random inchworm
                self.worm = Inchworm(pygame.Rect(random.randint(50, 550), YRES - 10, 4, 1), self)


		
	def tick(self, time):
		"""
			Update the logic of the game.
		"""
		#only keep playing rain sound for 100ms
		if self.playingRainSound:
                        self.rainSoundCount -= time
                        if self.rainSoundCount <= 0:
                                self.rainSoundCount = 100
                                if self.playingRainSound == True:
                                        self.playingRainSound = False
                                        self.rainSound.stop()
                #only keep playing bug sound for 100ms
                if self.playingBugSound:
                        self.bugSoundCount -= time
                        if self.bugSoundCount <= 0:
                                self.bugSoundCount = 100
                                if self.playingBugSound == True:
                                        self.playingBugSound = False
                                        self.bugSound.stop()

                #patches contains a list of where water hit this frame
                patches = []
                #rain logic for each drop
                for rain in self.drops:
                        rain.setWind(self.windSpeed)
                        rain.update(time)
                        #check if it hits any bugs
                        for bug in self.bugs:
                                if distance(bug.trueX, bug.trueY, rain.x, rain.y) < 2:
                                        bug.confidence = 0
                                elif distance(bug.trueX, bug.trueY, rain.endX, rain.endY) < 2:
                                        bug.confidence = 0
                        #check if it hits the ground
                        if rain.y > YRES - 8 and rain.shouldRemove == False:
                                self.addVine(rain.x)
                                patches.append(rain.x)
                                #handle particle effects for splash
                                if rain.shouldExplode:
                                        rain.explode()
                                        if self.worm.color == self.worm.black:
                                                if math.sqrt(math.pow((self.worm.x + 2) - rain.x, 2)) < 3:
                                                        self.worm.makeAngry()
                                self.rainSoundCount = 100
                                #play rain sound if not playing
                                if self.playingRainSound == False:
                                        self.rainSound.play(loops=-1)
                                        self.playingRainSound = True
                        #remove rain from game
                        elif rain.shouldRemove == True:
                                self.drops.remove(rain)

                #update all the plants in the game
		for t in self.turtles:
                        t.setWind(self.windSpeed)
                        for p in patches:
                                #check if watered
                                if math.sqrt(math.pow(p - t.startX, 2)) < 10:
                                        t.watered = True
                                        break
                        t.update(time)
                        #check if should be removed
                        if t.shouldRemove == True:
                                self.bugs.append(Bug(pygame.Rect(t.startX, YRES - 10, 1, 1)))
                                self.turtles.remove(t)
                                for bug in self.bugs:
                                        if bug.tTarget == t:
                                                bug.tTarget = None
                                                bug.target = None
                                
                #update bug logic
                for bug in self.bugs:
                        bug.target = None
                        bug.tTarget = None
                        dist = 9000
                        #check for nearest plant to target
                        for t in self.turtles:
                                d = abs(bug.trueX - t.startX)
                                if d < dist:
                                        dist = d
                                        bug.tTarget = t
                                        bug.target = Vec2(bug.tTarget.startX, YRES - 15)
                        if bug.tTarget != None:
                                dist = distance(bug.target.x, bug.target.y, bug.trueX, bug.trueY)
                                #if a bug is within 10 pixels of target, damage the target plant
                                if dist < 10:
                                        bug.tTarget.waterLevel = max(bug.tTarget.waterLevel - 0.001, 0.0)
                                        self.bugSoundCount = 100
                                        if self.playingBugSound == False and bug.confidence <= 0:
                                                self.bugSound.play(loops=-1)
                                                self.playingBugSound = True
                        bug.update(time)
                #update inchworm logic
                self.worm.update(time)

		#handle controls
                if self.holdLeft == True:
                        if self.hero.energy >= 0.5:
                                self.windSpeed = max(self.windSpeed - 0.1, -3.0)
                                self.hero.energy -= 0.01
                        else:
                                self.holdLeft = False
                elif self.holdRight == True:
                        if self.hero.energy >= 0.5:
                                self.windSpeed = min(self.windSpeed + 0.1, 3.0)
                                self.hero.energy -= 0.01
                        else:
                                self.holdRight = False
                if self.raining == True:
                        if self.hero.energy >= 3.0:
                                self.rain()
                                self.hero.energy -= 3.0
                        else:
                                self.raining = False

                if self.lightningCounter > 0:
                        self.lightningCounter -= time
                #update clouds
		for cloud in self.foreClouds:
                        cloud.setWind(self.windSpeed, 0)
                        cloud.update(time)
                #update player logic
                self.hero.setWind(self.windSpeed, 0)
                self.hero.update(time)

                

                
		
	def draw(self, surface):
		"""
			Draw the game objects to screen.
		"""
		#sky background
		pygame.draw.rect(surface, [46, 118, 222], pygame.Rect(0, 0, XRES, YRES))


                #rain                
                for rain in self.drops:
                        rain.draw(surface)
                #plants
                for turtle in self.turtles:
                        turtle.draw(surface)
                
                #bugs
                for bug in self.bugs:
                        bug.draw(surface)

                #grass          
		pygame.draw.line(surface, [0, 100, 0], [0, YRES - 4], [XRES, YRES - 4], 10)

		for cloud in self.foreClouds:
                        #draw cloud
                        cloud.draw(surface)
                        #draw shadow
                        pygame.draw.aaline(surface, [50, 50, 50, 128], [cloud.lowestX + cloud.trueX - 50, YRES - 8], [cloud.highestX + cloud.trueX - 50, YRES - 8])
                #draw hero shadow
                pygame.draw.aaline(surface, [50, 50, 50, 128], [self.hero.lowestX + self.hero.trueX - 50, YRES - 8], [self.hero.highestX + self.hero.trueX - 50, YRES - 8])
                #draw hero
                self.hero.draw(surface)

                #draw inchworm
                self.worm.draw(surface)

                #drawn lightning effect
		if self.lightningCounter > 0:
                        if self.lightningCounter >= 180:
                                pygame.draw.rect(surface, [255, 255, 255, 100], pygame.Rect(0, 0, XRES, YRES))
                        if self.lightningCounter >= 120 and self.lightningCounter < 180:
                                pygame.draw.rect(surface, [255, 255, 255, 200], pygame.Rect(0, 0, XRES, YRES))
                        if self.lightningCounter < 60:
                                pygame.draw.rect(surface, [255, 255, 255, 100], pygame.Rect(0, 0, XRES, YRES))

                

        def rain(self):
                """
                        Add a raindrop.
                """
                self.drops.append(RainDrop(pygame.Rect(random.randint(int(self.hero.trueX) + self.hero.lowestX, int(self.hero.trueX) + self.hero.highestX), int(self.hero.trueY - 5 + self.hero.highestY), 1, 1)))


        def addVine(self, x):
                """
                        Add a new vine at position x only if there are none close by.
                        "Close by" is determined by the spacing variable
                """
                nearby = False
                if x > 0 and x < XRES:
                        for vine in self.turtles:
                                if math.sqrt(math.pow(x - vine.startX, 2)) < self.spacing:
                                        nearby = True
                                        break
                        if nearby == False:
                                self.turtles.append(Turtle(RuleSystem("data" + os.sep + "lsystems" + os.sep + "vine.txt"), x, YRES - 8, 90, 200))

		
	def handleEvent(self, event):
		"""
			Handles any events generated by the user.
		"""
		#quit when the user clicks the "X"
		if event.type == pygame.QUIT:
			sys.exit()
		elif event.type == KEYDOWN:
                        if event.key == K_SPACE:
                                #start raining
                                if self.hero.energy >= 3.0:
                                        self.raining = True
                        elif event.key == K_LEFT:
                                #blow left
                                if self.hero.energy >= 0.5:
                                        self.holdLeft = True
                                        self.holdRight = False
                        elif event.key == K_RIGHT:
                                #blow right
                                if self.hero.energy >= 0.5:
                                        self.holdRight = True
                                        self.holdLeft = False
                        elif event.key == K_RETURN:
                                #do lightning (if energy available)
                                if self.hero.energy >= 50.0:
                                        thunder = pygame.mixer.Sound(load_sound("thunder.ogg"))
                                        thunder.play()
                                        self.hero.energy -= 50.0
                                        self.lightningCounter = 180
                                        for bug in self.bugs:
                                                bug.resetConfidence()
                        elif event.key == K_ESCAPE:
                                #delete objects if the user pressed escape
                                self.foreClouds = []
                                self.backClouds = []
                                self.drops = []
                                self.raining = False
                                self.rainCount = 0
                                self.turtles = []
                                self.bugs = []
                                self.windSpeed = 2.3
                                self.holdLeft = False
                                self.holdRight = False
                                self.lightningCounter = 0
                                self.rainSound.stop()
                                self.bugSound.stop()
                                self.bugSoundCount = 100
                                self.rainSoundCount = 100
                                self.playingRainSound = False
                                self.playingBugSound = False
                                self.controller.getScene(0).hc = self.hero
                                self.controller.changeScene(0)
                elif event.type == KEYUP:
                        #stop raining
                        if event.key == K_SPACE:
                                self.raining = False
                        #stop blowing left
                        elif event.key == K_LEFT:
                                self.holdLeft = False
                        #stop blowing right
                        elif event.key == K_RIGHT:
                                self.holdRight = False

            
