#Inchworm.py

import pygame

from engine.AtlanSprite import AtlanSprite
from Bug import Bug

from common.Utility import *


class Inchworm(AtlanSprite):
    """
        Worm like critter who moves along the bottom of the screen.
        If it gets wet it lays an egg which hatches a fly.
    """


    def __init__(self, myBounds, m):
        """
            Create a new Inchworm at the top x,y corner of the rect
            specified by bounds.
            m = reference to CloudyDay instance from CloudDay.py
        """
        AtlanSprite.__init__(self)
        self.bounds = myBounds
        self.x = self.bounds.x
        self.y = self.bounds.y
        self.frame = 0
        self.animationTimer = 500
        self.black = [0, 0, 0, 255]
        self.red = [100, 0, 0, 255]
        self.color = self.black
        self.eggTimer = 5000
        self.master = m
        

    


    def draw(self, surface):
        """
            Has two frames of animation - flat, and arched.
            Draw based on current color.
        """
        if self.frame == 0:
            #flat
            pygame.draw.line(surface, self.color, [self.x, self.y], [self.x + 8, self.y], 3)
        else:
            #arched
            pygame.draw.line(surface, self.color, [self.x, self.y], [self.x + 2, self.y], 3)
            pygame.draw.line(surface, self.color, [self.x + 2, self.y], [self.x + 2, self.y - 2], 3)
            pygame.draw.line(surface, self.color, [self.x + 2, self.y - 2], [self.x + 4, self.y - 2], 3)
            pygame.draw.line(surface, self.color, [self.x + 4, self.y - 2], [self.x + 4, self.y], 3)
            pygame.draw.line(surface, self.color, [self.x + 4, self.y], [self.x + 6, self.y], 3)

    def makeAngry(self):
        """
            Make the color red and start the egg laying process.
        """
        self.color = self.red

    def update(self, delta):
        """
            Update the internal logic of this Inchworm.
        """
        #handle animation
        #move 2 pixels whenever it switches to frame 0
        self.animationTimer -= delta
        if self.animationTimer < 0:
            self.frame += 1
            self.animationTimer = 500
            if self.frame > 1:
                self.frame = 0
                if self.color == self.black:
                    self.x -= 2
        #lay egg after a certain time
        if self.color == self.red:
            self.frame = 1
            self.eggTimer -= delta
            if self.eggTimer <= 0:
                self.color = self.black
                self.eggTimer = 5000
                #tell the CloudyDay instance where to pug the new bug
                self.master.bugs.append(Bug(pygame.Rect(self.x - 4, self.y, 1, 1)))
        #wrap around the screen
        if self.x >= XRES + 5:
            self.x = -4
        elif self.x <= -5:
            self.x = XRES - 4

        
            
